-----------------------------------------------------------------------------------------------
-- Client Lua Script for NSkillSwap
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- NSkillSwap Module Definition
-----------------------------------------------------------------------------------------------
local NSkillSwap = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
local kcrSelectedText = ApolloColor.new("UI_BtnTextHoloPressedFlyby")
local kcrNormalText = ApolloColor.new("UI_BtnTextHoloNormal")
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function NSkillSwap:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	o.tItems = {} -- keep track of all the list items
	o.wndSelectedListItem = nil -- keep track of which list item is currently selected

    return o
end

function NSkillSwap:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- NSkillSwap OnLoad
-----------------------------------------------------------------------------------------------
function NSkillSwap:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("NSkillSwap.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- NSkillSwap OnDocLoaded
-----------------------------------------------------------------------------------------------
function NSkillSwap:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "NSkillSwapForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		-- item list
		self.wndItemList = self.wndMain:FindChild("ItemList")
	    self.wndMain:Show(false, true)
		
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("nss", "OnNSkillSwapOn", self)
		
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("ToggleNSkillSwap", "OnNSkillSwapOn", self)
		
		--Debugging commands
		--Apollo.RegisterSlashCommand("ds", "dumpSave", self)
		--Apollo.RegisterSlashCommand("dr", "dumpRestore", self)
		--Apollo.RegisterSlashCommand("dt", "dumpTItems", self)
		
		-- Do additional Addon initialization here
		--Restore skill names, IDs are refreshed, but they seem to generally be refreshed in the same order they were added
		if Items ~= nil then
			for i=2, table.getn(Items), 2 do
				self:AddItem(Items[i-1], Items[i])
			end
			
			self.wndMain:SetAnchorOffsets(Items.left, Items.top, Items.right, Items.bottom)
		end
		
		
	end
end

-----------------------------------------------------------------------------------------------
-- NSkillSwap Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/nss"
function NSkillSwap:OnNSkillSwapOn(strCmd, strArg)
	--if strArg has something in it, it should be a number, 
	--but more specifically it should be a valid id
	--since all numbers under the length of tItems and greater than 0 are valid we can check against that'
	
	if tonumber(strArg) ~= nil then
		--from the lua documentation: "Equality first compares the types of its operands. If they are different, the result is nil."
		--This is apparently bs, it just throws a fatal error
		if tonumber(strArg) > 0 and tonumber(strArg) <= table.getn(self.tItems) then
			self:Toggle(tonumber(strArg))
		end
	else
		if self.wndMain:IsVisible() then
			self.wndMain:Close()
		else
			self.wndMain:Invoke()
		end	
	end
end

function NSkillSwap:Toggle(id)
	--ID checking is done before this function is called
	--Most, if not all of this function is taken from SpellSwap
	
	local skill1 = self.tItems[id]:FindChild("Skill1")
	local skillID1 = self:GetSkillId(skill1:GetText(skill1))
	local skill2 = self.tItems[id]:FindChild("Skill2")
	local skillID2 = self:GetSkillId(skill2:GetText(skill2))
	
	--Print("Skill 1: " .. skill1:GetText(skill1))
	--Print("Skill 2: " .. skill2:GetText(skill2))
	--Print("Skill ID 1: " .. skillID1)
	--Print("Skill ID 2: " .. skillID2)
		
	local oldid, newid, laskey
	local skillsfound = 0
	
	local luaAbilities = Apollo.GetAddon("Abilities")
	if luaAbilities and luaAbilities.wndMain then
		luaAbilities:OnCloseFinal()
	end
	local las = ActionSetLib.GetCurrentActionSet()
	
	for keybind,skill in pairs(las) do
		if skill == skillID1 then
			oldid = skillID1
			newid = skillID2
			laskey = keybind
			skillsfound = 1
			break
		elseif skill == skillID2 then
			oldid = skillID2
			newid = skillID1
			laskey = keybind
			skillsfound = 1
			break
		end
	end
	
	--Print("Skills found: " .. skillsfound)
	
	if skillsfound == 1 then
		local abs = AbilityBook.GetAbilitiesList()
		local nTier
		for k,v in pairs(abs) do
			if v.nId == oldid then
				nTier = v.nCurrentTier
				break
			end
		end
		AbilityBook.UpdateSpellTier(oldid,1)
		AbilityBook.UpdateSpellTier(newid,nTier)
		las[laskey] = newid
		local tResult = ActionSetLib.RequestActionSetChanges(las)
		if tResult.eResult == ActionSetLib.CodeEnumLimitedActionSetResult.Ok then
			--All went well
		end
	end
end

--The below functions are credited to SpellSwap
--I did not write them
function NSkillSwap:GetAbilitiesList()
	if self.abilitiesList == nil then
		self.abilitiesList = AbilityBook.GetAbilitiesList()
	end
	return self.abilitiesList
end

function NSkillSwap:GetSkillId(skillName)
	local abilities = self:GetAbilitiesList()
	if not abilities then
		return
	end
	for _, v in pairs(abilities) do
		if v.strName == skillName then
			return v.nId
		end
	end
	return nil
end

-----------------------------------------------------------------------------------------------
-- Save and Restore Functions
-----------------------------------------------------------------------------------------------

function NSkillSwap:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end
	
	--Attempting to work with multidimensional/layered tables didn't seem to work well
	--Converted to flat table, provided the appropriate conversions in OnDocLoaded
	--Tried putting the conversions in onRestore, but that fired before OnDocLoaded and caused UI glitches
	--Added bonus of "fixing" some ID errors due to the flat readd in onDocLoaded
	
	local save = {}
	local length = table.getn(self.tItems)

	--skill names
	for i=2, length*2, 2 do
		save[i-1] = self.tItems[i/2]:FindChild("Skill1"):GetText(self.tItems[i/2]:FindChild("Skill1"))
		save[i] = self.tItems[i/2]:FindChild("Skill2"):GetText(self.tItems[i/2]:FindChild("Skill2"))
	end
	
	--no idea how this will affect hierarchy of the table, such as table length or anything
	save.left, save.top, save.right, save.bottom = self.wndMain:GetAnchorOffsets()
	
	return save
end

function NSkillSwap:OnRestore(eLevel, tData)
    -- just store this and use it later, in onDocLoaded
    Items = tData
end

-----------------------------------------------------------------------------------------------
-- NSkillSwapForm Functions
-----------------------------------------------------------------------------------------------
function NSkillSwap:OnAdd()
	self:AddItem()
end

function NSkillSwap:OnClose()
	self.wndMain:Close()
end

function NSkillSwap:OnRemoveSkill(wndHandler, wndControl, eMouseButton)
	self:RemoveItem(wndHandler:GetId())
end

--Taken from MrFancyPants
function NSkillSwap:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "nSkillSwap", {"ToggleNSkillSwap", "", ""})
end

-----------------------------------------------------------------------------------------------
-- ItemList Functions
-----------------------------------------------------------------------------------------------

-- add an item into the item list
function NSkillSwap:AddItem(skill1, skill2)
	-- load the window item for the list item
	local wnd = Apollo.LoadForm(self.xmlDoc, "ListItem", self.wndItemList, self)
	--used for deleting later on
	local i = table.getn(self.tItems) + 1
	wnd:SetName(i)
	
	-- keep track of the window item created
	--does not store items/gets cleared of items stored some time after the OnRestore function
	self.tItems[i] = wnd
	--Print("Id: " .. wnd:GetName())
	--Print("Size of tItems: " .. table.getn(self.tItems))
	
	--Default text
	if skill1 == nil then
		skill1 = "Skill 1"
	end
	if skill2 == nil then
		skill2 = "Skill 2"
	end
	
	wnd:FindChild("ID"):SetText(i)
	wnd:FindChild("Skill1"):SetText(skill1)
	wnd:FindChild("Skill2"):SetText(skill2)
	
	--I have no idea what this line does
	wnd:SetData(i)
	
	--update display, will stack on top of each other if not updated
	self.wndItemList:ArrangeChildrenVert()
end

function NSkillSwap:RemoveItem(id)
	for i=table.getn(self.tItems), 1, -1 do
		if id == self.tItems[i]:FindChild("DeleteButton"):GetId() then
			--self:printArray(self.tItems)
			
			local d = self.wndItemList:FindChild(self.tItems[i]:GetName())
			d:Destroy()
			table.remove(self.tItems, i)
			
			for j=i, table.getn(self.tItems) do
				self.tItems[j]:FindChild("ID"):SetText(j)
			end
			
			--self:printArray(self.tItems)
			self.wndItemList:ArrangeChildrenVert()
			return
		end
	end	
end

-----------------------------------------------------------------------------------------------
--The following four items are for debugging purposes only
-----------------------------------------------------------------------------------------------

function NSkillSwap:dumpTItems()
	if self.tItems ~= nil then
		Print("tItems Exists")
		for i = 1, table.getn(self.tItems) do
			if self.tItems[i] ~= nil then
				Print("tItem[" .. i .. "] exists, name: " .. self.tItems[i]:GetName())
				local wnd1 = self.tItems[i]:FindChild("Skill1")
				local wnd2 = self.tItems[i]:FindChild("Skill2")
			
				if wnd1 ~= nil then
					Print("Skill 1 text: " .. wnd1:GetText(wnd1))
				end
				if wnd2 ~= nil then
					Print("Skill 2 text: " .. wnd2:GetText(wnd2))
				end
			end
		end
	end
end

function NSkillSwap:dumpSave()
	eLevel = nil
	
	local save = self:OnSave(eLevel)
	if save == nil then
		Print("Save is nil")
	else
		Print("Save is not nil, length: " .. table.getn(save))
		for i = 1, table.getn(save) do
			Print("Name: " .. save[i])
		end
	end
end

function NSkillSwap:dumpRestore()
	if Items == nil then
		Print("Items is nil")
	else
		Print("Items is not nil, length: " .. table.getn(Items))
		for i = 1, table.getn(Items) do
			Print("Name: " .. Items[i])
		end
	end
end

function NSkillSwap:printArray(arr)
	Print("size of array: "..table.getn(arr))
	
	for i=1,table.getn(arr) do
		Print("Array["..i.."] (name: " .. arr[i]:GetName() .. ") = " .. arr[i]:FindChild("Skill1").GetText(arr[i]:FindChild("Skill1")) .. ", " .. arr[i]:FindChild("Skill2").GetText(arr[i]:FindChild("Skill2")))
	end
end

-----------------------------------------------------------------------------------------------
-- NSkillSwap Instance
-----------------------------------------------------------------------------------------------
local NSkillSwapInst = NSkillSwap:new()
NSkillSwapInst:Init()
